=== modified file 'database/schema/fti.py'
--- database/schema/fti.py	2011-09-21 03:09:33 +0000
+++ database/schema/fti.py	2011-09-21 12:45:39 +0000
@@ -1,6 +1,6 @@
 #!/usr/bin/python -S
 #
-# Copyright 2009 Canonical Ltd.  This software is licensed under the
+# Copyright 2009-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 #
 # This modules uses relative imports.
@@ -11,34 +11,41 @@
 """
 __metaclass__ = type
 
-import _pythonpath
-
 from distutils.version import LooseVersion
+from optparse import OptionParser
 import os.path
-from optparse import OptionParser
 import subprocess
 import sys
 from tempfile import NamedTemporaryFile
 from textwrap import dedent
 import time
 
+import _pythonpath
 import psycopg2.extensions
+import replication.helpers
 
 from canonical.config import config
 from canonical.database.postgresql import ConnectionString
 from canonical.database.sqlbase import (
-    connect, ISOLATION_LEVEL_AUTOCOMMIT, ISOLATION_LEVEL_READ_COMMITTED,
-    quote, quote_identifier)
-from canonical.launchpad.scripts import logger, logger_options, db_options
-
-import replication.helpers
+    connect,
+    ISOLATION_LEVEL_AUTOCOMMIT,
+    ISOLATION_LEVEL_READ_COMMITTED,
+    quote,
+    quote_identifier,
+    )
+from canonical.launchpad.scripts import (
+    db_options,
+    logger,
+    logger_options,
+    )
 
 # Defines parser and locale to use.
 DEFAULT_CONFIG = 'default'
 
 PGSQL_BASE = '/usr/share/postgresql'
 
-A, B, C, D = 'ABCD' # tsearch2 ranking constants
+# tsearch2 ranking constants:
+A, B, C, D = 'ABCD'
 
 # This data structure defines all of our full text indexes.  Each tuple in the
 # top level list creates a 'fti' column in the specified table.
@@ -224,7 +231,7 @@
     # Create the trigger
     columns_and_weights = []
     for column, weight in qcolumns:
-        columns_and_weights.extend( (column, weight) )
+        columns_and_weights.extend((column, weight))
 
     sql = """
         CREATE TRIGGER tsvectorupdate BEFORE UPDATE OR INSERT ON %s
@@ -256,7 +263,9 @@
 def liverebuild(con):
     """Rebuild the data in all the fti columns against possibly live database.
     """
-    batch_size = 50 # Update maximum of this many rows per commit
+    # Update number of rows per transaction.
+    batch_size = 50
+
     cur = con.cursor()
     for table, ignored in ALL_FTI:
         table = quote_identifier(table)
@@ -316,13 +325,14 @@
         p = subprocess.Popen(
             cmd.split(' '), stdin=subprocess.PIPE,
             stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+        tsearch2_sql = open(tsearch2_sql_path).read()
         out, err = p.communicate(
-            "SET client_min_messages=ERROR; CREATE SCHEMA ts2;"
-            + open(tsearch2_sql_path).read().replace('public;','ts2, public;'))
+            "SET client_min_messages=ERROR; CREATE SCHEMA ts2;" +
+            tsearch2_sql.replace('public;', 'ts2, public;'))
         if p.returncode != 0:
-            log.fatal('Error executing %s:', cmd)
+            log.fatal("Error executing %s:", cmd)
             log.debug(out)
-            sys.exit(rv)
+            sys.exit(p.returncode)
 
     # Create ftq helper and its sibling _ftq.
     # ftq(text) returns a tsquery, suitable for use querying the full text
@@ -363,7 +373,8 @@
         # Strip ! characters inside and at the end of a word
         query = re.sub(r"(?u)(?<=\w)[\!]+", " ", query)
 
-        # Now that we have handle case sensitive booleans, convert to lowercase
+        # Now that we have handled case-sensitive booleans, convert to
+        # lowercase.
         query = query.lower()
 
         # Convert foo-bar to ((foo&bar)|foobar) and foo-bar-baz to
@@ -454,7 +465,7 @@
         p = plpy.prepare("SELECT to_tsquery('%s', $1) AS x", ["text"])
         query = plpy.execute(p, [query], 1)[0]["x"]
         return query or None
-        """  % configuration
+        """ % configuration
     sexecute(con, r"""
         CREATE OR REPLACE FUNCTION ts2._ftq(text) RETURNS text AS %s
         LANGUAGE plpythonu IMMUTABLE
@@ -569,7 +580,7 @@
     We know this by looking in our cache to see what the previous
     definitions were, and the --force command line argument
     '''
-    current_columns = repr(sorted(columns)) # Convert to a string
+    current_columns = repr(sorted(columns))
 
     existing = execute(
         con, "SELECT columns FROM FtiCache WHERE tablename=%(table)s",
@@ -621,6 +632,7 @@
 # Files for output generated for slonik(1). None if not a Slony-I install.
 slonik_sql = None
 
+
 def main():
     parser = OptionParser()
     parser.add_option(
@@ -706,5 +718,3 @@
 
 if __name__ == '__main__':
     sys.exit(main())
-
-

=== added file 'database/schema/patch-2208-87-1.sql'
--- database/schema/patch-2208-87-1.sql	1970-01-01 00:00:00 +0000
+++ database/schema/patch-2208-87-1.sql	2011-09-21 02:05:56 +0000
@@ -0,0 +1,84 @@
+-- Copyright 2011 Canonical Ltd.  This software is licensed under the
+-- GNU Affero General Public License version 3 (see the file LICENSE).
+
+SET client_min_messages=ERROR;
+
+CREATE OR REPLACE FUNCTION update_transitively_private(
+    start_branch int,
+    _root_branch int = NULL,
+    _root_transitively_private boolean = NULL) RETURNS VOID
+LANGUAGE plpgsql VOLATILE SECURITY DEFINER SET search_path TO PUBLIC AS
+$$
+DECLARE
+    root_transitively_private boolean := _root_transitively_private;
+    root_branch int := _root_branch;
+BEGIN
+    IF root_transitively_private IS NULL THEN
+        -- We can't just trust the transitively_private flag of the
+        -- branch we are stacked on, as if we are updating multiple
+        -- records they will be updated in an indeterminate order.
+        -- We need a recursive query.
+        UPDATE Branch SET transitively_private = (
+            WITH RECURSIVE stacked_branches AS (
+                SELECT
+                    top_branch.id, top_branch.stacked_on, top_branch.private
+                FROM Branch AS top_branch
+                WHERE top_branch.id = start_branch
+                UNION ALL
+                SELECT
+                    sub_branch.id, sub_branch.stacked_on, sub_branch.private
+                FROM stacked_branches, Branch AS sub_branch
+                WHERE
+                    stacked_branches.stacked_on = sub_branch.id
+                    AND stacked_branches.stacked_on != start_branch
+                    -- Shortcircuit. No need to recurse if already private.
+                    AND stacked_branches.private IS FALSE
+                    )
+            SELECT COUNT(*) > 0
+            FROM stacked_branches
+            WHERE private IS TRUE)
+        WHERE Branch.id = start_branch
+        RETURNING transitively_private INTO root_transitively_private;
+        root_branch := start_branch;
+    ELSE
+        -- Now we have calculated the correct transitively_private flag
+        -- we can trust it.
+        UPDATE Branch SET
+            transitively_private = GREATEST(private, root_transitively_private)
+        WHERE id = root_branch;
+    END IF;
+
+    -- Recurse to branches stacked on this one.
+    PERFORM update_transitively_private(
+        start_branch, id, GREATEST(private, root_transitively_private))
+    FROM Branch WHERE stacked_on = root_branch AND id != start_branch;
+END;
+$$;
+
+COMMENT ON FUNCTION update_transitively_private(int, int, boolean) IS
+'A branch is transitively private if it is private or is stacked on any transitively private branches.';
+
+CREATE OR REPLACE FUNCTION maintain_transitively_private() RETURNS TRIGGER
+LANGUAGE plpgsql VOLATILE AS
+$$
+BEGIN
+    IF TG_OP = 'UPDATE' THEN
+        IF (NEW.stacked_on IS NOT DISTINCT FROM OLD.stacked_on
+            AND NEW.private IS NOT DISTINCT FROM OLD.private) THEN
+            RETURN NULL;
+        END IF;
+    END IF;
+    PERFORM update_transitively_private(NEW.id);
+    RETURN NULL;
+END;
+$$;
+
+COMMENT ON FUNCTION maintain_transitively_private() IS
+    'Trigger maintaining the Branch transitively_private column';
+
+CREATE TRIGGER maintain_branch_transitive_privacy_t
+    AFTER INSERT OR UPDATE ON Branch
+    FOR EACH ROW
+    EXECUTE PROCEDURE maintain_transitively_private();
+
+INSERT INTO LaunchpadDatabaseRevision VALUES (2208, 87, 1);

=== added file 'database/schema/patch-2208-89-1.sql'
--- database/schema/patch-2208-89-1.sql	1970-01-01 00:00:00 +0000
+++ database/schema/patch-2208-89-1.sql	2011-09-16 02:48:16 +0000
@@ -0,0 +1,8 @@
+-- Copyright 2011 Canonical Ltd.  This software is licensed under the
+-- GNU Affero General Public License version 3 (see the file LICENSE).
+SET client_min_messages=ERROR;
+
+CREATE INDEX branch__transitively_private__idx ON Branch(transitively_private);
+
+INSERT INTO LaunchpadDatabaseRevision VALUES (2208, 89, 1);
+

=== modified file 'database/schema/pending/prune-nonce.py'
--- database/schema/pending/prune-nonce.py	2011-09-20 03:10:08 +0000
+++ database/schema/pending/prune-nonce.py	2011-09-20 10:57:34 +0000
@@ -1,6 +1,6 @@
 #!/usr/bin/python -S
 #
-# Copyright 2009 Canonical Ltd.  This software is licensed under the
+# Copyright 2009-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 """Prune old nonces."""
@@ -8,13 +8,20 @@
 __metaclass__ = type
 __all__ = []
 
+from optparse import OptionParser
+
 import _pythonpath
 
-from optparse import OptionParser
-
-from canonical.database.sqlbase import connect, ISOLATION_LEVEL_AUTOCOMMIT
+from canonical.database.sqlbase import (
+    connect,
+    ISOLATION_LEVEL_AUTOCOMMIT,
+    )
 from canonical.launchpad.scripts import db_options
-from canonical.launchpad.scripts.logger import log, logger_options
+from canonical.launchpad.scripts.logger import (
+    log,
+    logger_options,
+    )
+
 
 def update_until_done(con, table, query, vacuum_every=100):
     log.info("Running %s" % query)

=== removed file 'database/schema/pending/update-translation-credits.py'
--- database/schema/pending/update-translation-credits.py	2011-09-20 03:11:38 +0000
+++ database/schema/pending/update-translation-credits.py	1970-01-01 00:00:00 +0000
@@ -1,111 +0,0 @@
-#!/usr/bin/python -S
-#
-# Copyright 2009 Canonical Ltd.  This software is licensed under the
-# GNU Affero General Public License version 3 (see the file LICENSE).
-
-"""Populate some new columns on the Person table."""
-
-__metaclass__ = type
-__all__ = []
-
-import _pythonpath
-
-from optparse import OptionParser
-import sys
-
-from canonical.database.sqlbase import connect, ISOLATION_LEVEL_AUTOCOMMIT
-from canonical.launchpad.scripts import db_options
-from canonical.launchpad.scripts.logger import log, logger_options
-
-def update_until_done(con, table, query, vacuum_every=100):
-    log.info("Running %s" % query)
-    loops = 0
-    total_rows = 0
-    cur = con.cursor()
-    while True:
-        loops += 1
-        cur.execute(query)
-        rowcount = cur.rowcount
-        total_rows += rowcount
-        log.debug("Updated %d" % total_rows)
-        if loops % vacuum_every == 0:
-            log.debug("Vacuuming %s" % table)
-            cur.execute("VACUUM %s" % table)
-        if rowcount <= 0:
-            log.info("Done")
-            return
-
-parser = OptionParser()
-logger_options(parser)
-db_options(parser)
-options, args = parser.parse_args()
-
-con = connect(isolation=ISOLATION_LEVEL_AUTOCOMMIT)
-
-#  People have so far updated translation credits, often mis-crediting people,
-#  or removing credits to upstream translators: we want to disable all of these
-#  translation, so automatic handling will pick up from now on.
-update_until_done(con, 'posubmission', """
-    UPDATE posubmission SET active=FALSE WHERE id IN (
-        SELECT posubmission.id
-        FROM posubmission,
-             pomsgset,
-             potmsgset,
-             pomsgid
-        WHERE
-            posubmission.active IS TRUE AND
-            posubmission.pomsgset=pomsgset.id AND
-            potmsgset=potmsgset.id AND
-            primemsgid=pomsgid.id AND
-            published IS NOT TRUE AND
-            (msgid='translation-credits' OR
-             msgid='translator-credits' OR
-             msgid='translator_credits' OR
-             msgid=E'_: EMAIL OF TRANSLATORS\nYour emails' OR
-             msgid=E'_: NAME OF TRANSLATORS\nYour names')
-        LIMIT 200
-        )
-    """)
-
-# Set any existing inactive published translations as active
-update_until_done(con, 'posubmission', """
-    UPDATE posubmission SET active=TRUE WHERE id IN (
-        SELECT posubmission.id
-        FROM posubmission,
-             pomsgset,
-             potmsgset,
-             pomsgid
-        WHERE
-            posubmission.active IS FALSE AND
-            posubmission.pomsgset=pomsgset.id AND
-            pomsgset.potmsgset=potmsgset.id AND
-            potmsgset.primemsgid=pomsgid.id AND
-            posubmission.published IS TRUE AND
-            (msgid='translation-credits' OR
-             msgid='translator-credits' OR
-             msgid='translator_credits' OR
-             msgid=E'_: EMAIL OF TRANSLATORS\nYour emails' OR
-             msgid=E'_: NAME OF TRANSLATORS\nYour names')
-        LIMIT 200
-        )
-    """)
-
-# Remove reviewer, date_reviewed from all translation credit POMsgSets
-update_until_done(con, 'pomsgset', """
-    UPDATE POMsgSet SET reviewer=NULL, date_reviewed=NULL
-    WHERE id IN (
-        SELECT POMsgSet.id
-        FROM POMsgSet, POTMsgSet, POMsgId
-        WHERE
-            POMsgSet.reviewer IS NOT NULL AND
-            POMsgSet.potmsgset=POTMsgSet.id AND
-            POTMsgSet.primemsgid=POMsgId.id AND
-            (msgid='translation-credits' OR
-            msgid='translator-credits' OR
-            msgid='translator_credits' OR
-            msgid=E'_: EMAIL OF TRANSLATORS\nYour emails' OR
-            msgid=E'_: NAME OF TRANSLATORS\nYour names')
-        LIMIT 200
-        )
-    """)
-

=== modified file 'database/schema/security.cfg'
--- database/schema/security.cfg	2011-09-22 09:34:57 +0000
+++ database/schema/security.cfg	2011-09-23 08:44:17 +0000
@@ -68,6 +68,7 @@
 public.sha1(text)                          = EXECUTE
 public.ulower(text)                        = EXECUTE
 public.update_database_disk_utilization()  =
+public.update_transitively_private(integer, integer, boolean) =
 public.valid_absolute_url(text)            = EXECUTE
 public.valid_branch_name(text)             = EXECUTE
 public.valid_bug_name(text)                = EXECUTE

=== added file 'lib/lp/code/model/tests/test_branch_privacy_triggers.py'
--- lib/lp/code/model/tests/test_branch_privacy_triggers.py	1970-01-01 00:00:00 +0000
+++ lib/lp/code/model/tests/test_branch_privacy_triggers.py	2011-09-21 02:05:56 +0000
@@ -0,0 +1,211 @@
+# Copyright 2011 Canonical Ltd.  This software is licensed under the
+# GNU Affero General Public License version 3 (see the file LICENSE).
+
+"""Tests that the triggers to maintain the branch table transitively_private
+   column work correctly."""
+
+__metaclass__ = type
+
+import unittest
+
+from canonical.database.sqlbase import cursor
+from canonical.testing.layers import LaunchpadZopelessLayer
+
+
+class BranchPrivacyTriggersTestCase(unittest.TestCase):
+    layer = LaunchpadZopelessLayer
+
+    def setUp(self):
+        LaunchpadZopelessLayer.switchDbUser('testadmin')
+        self.branch_ids = dict()
+
+    def createBranches(self):
+        # Create branches 1, 2, 3, 4, 5
+        # 3 is private
+        # 4 stacked on 3 at insert time
+        # 2 stacked on 5
+        cur = cursor()
+        for x in range(6):
+            is_private = 'True' if x == 3 else 'False'
+            if x == 4:
+                stacked_on = self.branch_ids[3]
+            else:
+                stacked_on = 'NULL'
+            cur.execute("""
+                INSERT INTO Branch (
+                    name, private, stacked_on, owner, registrant, branch_type
+                    )
+                    VALUES ('branch%d', %s, %s, 1, 1, 1)
+                    RETURNING id
+                """ % (x, is_private, stacked_on))
+            branch_id = cur.fetchone()[0]
+            self.branch_ids[x] = branch_id
+        self.updateStackedOnForBranch(2, 5)
+
+    def check_branch_privacy(self, branch_num, expected_private,
+                             expected_transitively_private):
+        # Check branch_num has the expected privacy attributes.
+        is_private = 'True' if expected_private else 'False'
+        is_transitively_private = ('True'
+            if expected_transitively_private else 'False')
+        cur = cursor()
+        cur.execute("""
+            SELECT COUNT(*)
+            FROM Branch
+            WHERE id = %s
+                AND private is %s
+                AND transitively_private is %s
+            """ % (self.branch_ids[branch_num],
+                   is_private, is_transitively_private))
+        self.failUnlessEqual(cur.fetchone()[0], 1)
+
+    def updateStackedOnForBranch(self, branch_num, stacked_on):
+        # Update the stacked_on attribute for a branch.
+        cur = cursor()
+        if stacked_on:
+            stacked_on = self.branch_ids[stacked_on]
+        else:
+            stacked_on = 'NULL'
+        cur.execute("""
+            UPDATE Branch
+            SET stacked_on = %s
+            WHERE id = %s
+            """ % (stacked_on, self.branch_ids[branch_num]))
+
+    def updatePrivacyForBranch(self, branch_num, private):
+        # Update the private attribute for a branch.
+        is_private = 'True' if private else 'False'
+        cur = cursor()
+        cur.execute("""
+            UPDATE Branch
+            SET private = %s
+            WHERE id = %s
+            """ % (is_private, self.branch_ids[branch_num]))
+
+    def testInitialBranches(self):
+        # Branch inserts invokes the trigger correctly.
+        self.createBranches()
+        self.check_branch_privacy(1, False, False)
+        self.check_branch_privacy(3, True, True)
+        self.check_branch_privacy(4, False, True)
+
+    def testSetStackedOn(self):
+        # Stack 1 on 3 which is private.
+        # 1 should be transitively private.
+        self.createBranches()
+        self.updateStackedOnForBranch(1, 3)
+        self.check_branch_privacy(1, False, True)
+
+    def testMultiLevelStackedOn(self):
+        # 2 is stacked on 5 already.
+        # Make 5 private, stack 1 on 2.
+        # 1 and 2 should be transitively private.
+        self.createBranches()
+        self.updatePrivacyForBranch(5, True)
+        self.check_branch_privacy(5, True, True)
+        self.check_branch_privacy(2, False, True)
+        self.updateStackedOnForBranch(1, 2)
+        self.check_branch_privacy(1, False, True)
+        self.check_branch_privacy(2, False, True)
+
+    def testMultiLevelMultiStackedOn(self):
+        # 2 is stacked on 5 already.
+        # Stack 4 on 1.
+        # Make 5 private, stack 1 on 2.
+        # 1, 2 and 4 should be transitively private.
+        self.createBranches()
+        self.updateStackedOnForBranch(4, 1)
+        self.check_branch_privacy(4, False, False)
+        self.updatePrivacyForBranch(5, True)
+        self.check_branch_privacy(5, True, True)
+        self.check_branch_privacy(2, False, True)
+        self.updateStackedOnForBranch(1, 2)
+        self.check_branch_privacy(1, False, True)
+        self.check_branch_privacy(2, False, True)
+        self.check_branch_privacy(4, False, True)
+
+    def testRemoveStackedOn(self):
+        # 5 is private, 1 stacked on 5, so 1 is transitively private.
+        # Unstack 1.
+        # 1 should no longer be be transitively private.
+        self.createBranches()
+        self.updateStackedOnForBranch(1, 5)
+        self.updatePrivacyForBranch(5, True)
+        self.check_branch_privacy(1, False, True)
+        self.updateStackedOnForBranch(1, None)
+        self.check_branch_privacy(1, False, False)
+
+    def testSetPrivateTrue(self):
+        # Stack 1 on 5, make 5 private.
+        # 1 should be transitively private.
+        self.createBranches()
+        self.updateStackedOnForBranch(1, 5)
+        self.check_branch_privacy(1, False, False)
+        self.updatePrivacyForBranch(5, True)
+        self.check_branch_privacy(5, True, True)
+        self.check_branch_privacy(1, False, True)
+
+    def testMultiLevelSetPrivateTrue(self):
+        # 2 is stacked on 5 already.
+        # Stack 4 on 1, stack 1 on 2
+        # Make 5 private.
+        # 1, 2 and 4 should be transitively private.
+        self.createBranches()
+        self.updateStackedOnForBranch(4, 1)
+        self.check_branch_privacy(4, False, False)
+        self.updateStackedOnForBranch(1, 2)
+        self.check_branch_privacy(2, False, False)
+        self.updatePrivacyForBranch(5, True)
+        self.check_branch_privacy(5, True, True)
+        self.check_branch_privacy(1, False, True)
+        self.check_branch_privacy(2, False, True)
+        self.check_branch_privacy(4, False, True)
+
+    def testSetPrivateFalse(self):
+        # Make 5 private, stack 1 on 5.
+        # Make 5 public.
+        # 1 should not be transitively private.
+        self.createBranches()
+        self.updatePrivacyForBranch(5, True)
+        self.updateStackedOnForBranch(1, 5)
+        self.check_branch_privacy(1, False, True)
+        self.updatePrivacyForBranch(5, False)
+        self.check_branch_privacy(5, False, False)
+        self.check_branch_privacy(1, False, False)
+
+    def testMultlLevelSetPrivateFalse(self):
+        # 2 is stacked on 5 already.
+        # Make 5 private.
+        # Stack 4 on 1, stack 1 on 2
+        # Make 5 public.
+        # 1, 2 and 4 should not be transitively private.
+        self.createBranches()
+        self.updatePrivacyForBranch(5, True)
+        self.updateStackedOnForBranch(4, 1)
+        self.updateStackedOnForBranch(1, 2)
+        self.check_branch_privacy(1, False, True)
+        self.check_branch_privacy(2, False, True)
+        self.check_branch_privacy(4, False, True)
+        self.check_branch_privacy(5, True, True)
+        self.updatePrivacyForBranch(5, False)
+        self.check_branch_privacy(1, False, False)
+        self.check_branch_privacy(2, False, False)
+        self.check_branch_privacy(4, False, False)
+        self.check_branch_privacy(5, False, False)
+
+    def testLoopedStackedOn(self):
+        # It's possible, although nonsensical, for branch stackings to form a
+        # loop. e.g., branch A is stacked on branch B is stacked on branch A.
+        # 2 is stacked on 5 already.
+        # Stack 1 on 2, stack 5 on 1, completing a 1->2->5 loop.
+        # Stack 3 on 3, the trivial case.
+        self.createBranches()
+        self.updateStackedOnForBranch(1, 2)
+        self.updateStackedOnForBranch(5, 1)
+        self.updateStackedOnForBranch(3, 3)
+        self.updatePrivacyForBranch(5, True)
+        self.updatePrivacyForBranch(3, False)
+        self.check_branch_privacy(5, True, True)
+        self.check_branch_privacy(1, False, True)
+        self.check_branch_privacy(2, False, True)
+        self.check_branch_privacy(3, False, False)

